@@ -10,7 +10,7 @@
 #' @param nbreaks number of color breaks; should be one more than number of colors. If missing and \code{col} is specified, it is derived from that.
 #' @param breaks actual color breaks, or a method name used for \link[classInt]{classIntervals}.
 #' @param col colors to use for grid cells
-#' @param ... for \code{plot}, passed on to \code{image.stars}; for \code{image}, passed on to \code{image.default} or \code{rasterImage}.
+#' @param ... further arguments: for \code{plot}, passed on to \code{image.stars}; for \code{image}, passed on to \code{image.default} or \code{rasterImage}.
 #' @param key.pos integer; side to plot a color key: 1 bottom, 2 left, 3 top, 4 right; set to \code{NULL} to omit key. Ignored if multiple columns are plotted in a single function call. Default depends on plot size, map aspect, and, if set, parameter \code{asp}.
 #' @param key.width amount of space reserved for width of the key (labels); relative or absolute (using lcm)
 #' @param key.length amount of space reserved for length of the key (labels); relative or absolute (using lcm)
@@ -24,12 +24,13 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 
 	flatten = function(x, i) { # collapse all non-x/y dims into one, and select "layer" i
 		d = st_dimensions(x)
+		dxy = attr(d, "raster")$dimensions
 		dims = dim(x)
 		x = x[[1]]
-		aux = setdiff(names(dims), c("x", "y"))
-		newdims = c(dims[c("x", "y")], prod(dims[aux]))
+		aux = setdiff(names(dims), dxy)
+		newdims = c(dims[dxy], prod(dims[aux]))
 		dim(x) = newdims
-		st_as_stars(list(x[,,i]), dimensions = d[c("x", "y")])
+		st_as_stars(list(x[,,i]), dimensions = d[dxy])
 	}
 	key.pos.missing = missing(key.pos)
 	if (missing(nbreaks) && !missing(col))
@@ -37,14 +38,23 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 	opar = par()
 	dots = list(...)
 
+	#if (any(dim(x) == 1))
+	#	x = adrop(x)
+
 	if (join_zlim) {
-		breaks = get_breaks(x, breaks, nbreaks)
+		breaks = get_breaks(x, breaks, nbreaks, dots$logz)
 		nbreaks = length(breaks) # might be shorter than originally intended!
 	}
 
+	if (isTRUE(dots$logz) && !((has_raster(x) && (is_curvilinear(x) || has_rotate_or_shear(x))) || has_sfc(x)))
+		x = log10(x) # otherwise, defer log-transforming to sf::plot.sf
+
 	if (!missing(y))
 		stop("y argument should be missing")
 	if (has_raster(x)) {
+		dxy = attr(st_dimensions(x), "raster")$dimensions
+		loop = setdiff(names(dim(x)), dxy) # dimension (name) over which we loop, if any
+		x = aperm(x, c(dxy, loop))
 		zlim = if (join_zlim)
 				range(unclass(x[[1]]), na.rm = TRUE)
 			else
@@ -52,25 +62,25 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 		dims = dim(x)
 		if (downsample) {
 			n = dims * 0 + 1 # keep names
-			n[c("x", "y")] = get_downsample(dims)
+			n[dxy] = get_downsample(dims)
 			x = st_downsample(x, n)
 		}
-		if (length(dims) == 2 || dims[3] == 1 || !is.null(dots$rgb)) { ## ONE IMAGE:
+		if (length(dims) == 2 || dims[3] == 1 || (!is.null(dots$rgb) && is.numeric(dots$rgb))) { ## ONE IMAGE:
 			# set up key region
-			values = as.vector(x[[1]])
+			values = structure(x[[1]], dim = NULL) # array -> vector
 			if (! isTRUE(dots$add) && ! is.null(key.pos) && !all(is.na(values)) &&
 					(is.factor(values) || length(unique(na.omit(values))) > 1) &&
-					length(col) > 1 && is.null(dots$rgb)) { # plot key?
+					length(col) > 1 && is.null(dots$rgb) && !is_curvilinear(x)) { # plot key?
 				switch(key.pos,
 					layout(matrix(c(2,1), nrow = 2, ncol = 1), widths = 1, heights = c(1, key.width)),  # 1 bottom
 					layout(matrix(c(1,2), nrow = 1, ncol = 2), widths = c(key.width, 1), heights = 1),  # 2 left
 					layout(matrix(c(1,2), nrow = 2, ncol = 1), widths = 1, heights = c(key.width, 1)),  # 3 top
 					layout(matrix(c(2,1), nrow = 1, ncol = 2), widths = c(1, key.width), heights = 1)   # 4 right
 				)
-				if (is.factor(values)) {
+				if (is.factor(values))
 					.image_scale_factor(levels(values), col, key.pos = key.pos,
 						key.width = key.width, key.length = key.length, axes = axes, ...)
-				} else
+				else
 					.image_scale(values, col, breaks = breaks, key.pos = key.pos, 
 						key.width = key.width, key.length = key.length, axes = axes, ...) 
 			}
@@ -79,7 +89,8 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 			par(mar = c(axes * 2.1, axes * 2.1, 1 * !is.null(main), 0))
 
 			# plot the map:
-			image(x, ..., axes = axes, breaks = breaks, col = col)
+			image(x, ..., axes = axes, breaks = breaks, col = col, key.pos = key.pos, 
+				key.width = key.width, key.length = key.length, main = NULL)
 			if (!is.null(main))
 				title(main)
 
@@ -87,7 +98,7 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 			draw.key = !is.null(key.pos) && join_zlim
 			if (! draw.key)
 				key.pos = NULL
-			lt = .get_layout(st_bbox(x), dims[3], par("din"), 
+			lt = .get_layout(st_bbox(x), dims[3], par("din"),
 				if (join_zlim && key.pos.missing) -1 else key.pos, key.width)
 			title_size = if (is.null(main)) 
 					0
@@ -100,10 +111,10 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 				im = flatten(x, i)
 				if (! join_zlim) {
 					zlim = range(im[[1]], na.rm = TRUE)
-					br = get_breaks(im, breaks, nbreaks)
+					br = get_breaks(im, breaks, nbreaks, dots$logz)
 				} else
 					br = breaks
-				image(im, xlab = "", ylab = "", axes = axes, zlim = zlim, breaks = br, col = col, ...)
+				image(im, xlab = "", ylab = "", axes = axes, zlim = zlim, breaks = br, col = col, key.pos = NULL, main = NULL, ...)
 				if (!is.null(main)) {
 					if (length(main) == dims[3])
 						title(main[i])
@@ -115,7 +126,7 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 			for (i in seq_len(prod(lt$mfrow) - dims[3])) # empty panels:
 				plot.new()
 			if (draw.key) {
-				values = as.vector(x[[1]])
+				values = structure(x[[1]], dim = NULL)
 				if (is.factor(values))
 					.image_scale_factor(levels(values), col, key.pos = lt$key.pos,
 						key.width = key.width, key.length = key.length, axes = axes,...)
@@ -125,7 +136,9 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 			}
 		}
 	} else if (has_sfc(x)) {
-		plot(st_as_sf(x), ..., axes = axes)
+		if (key.pos.missing)
+			key.pos = -1
+		plot(st_as_sf(x), ..., key.pos = key.pos, key.length = key.length, key.width = key.width, reset = reset, axes = axes)
 	} else
 		stop("no raster, no features geometries: no default plot method set up yet!")
 	if (reset) {
@@ -135,11 +148,13 @@ plot.stars = function(x, y, ..., join_zlim = TRUE, main = names(x)[1], axes = FA
 	}
 }
 
-get_breaks = function(x, breaks, nbreaks) {
+get_breaks = function(x, breaks, nbreaks, logz = NULL) {
 	if (is.character(breaks)) { # compute breaks from values in x:
 		pdx = prod(dim(x[[1]]))
 		# take a regular sample from x[[1]]:
-		values = as.numeric(as.vector(x[[1]])[seq(1, pdx, length.out = min(pdx, 10000))])
+		values = as.numeric(structure(x[[1]], dim = NULL)[seq(1, pdx, length.out = min(pdx, 10000))])
+		if (isTRUE(logz))
+			values = log10(values)
 		n.unq = length(unique(na.omit(values)))
 		if (! all(is.na(values)) && n.unq > 1)
 			classInt::classIntervals(na.omit(values), min(nbreaks-1, n.unq), breaks, warnSmallN = FALSE)$brks
@@ -153,7 +168,7 @@ get_breaks = function(x, breaks, nbreaks) {
 #' @param band integer; which band (dimension) to plot
 #' @param attr integer; which attribute to plot
 #' @param asp numeric; aspect ratio of image
-#' @param rgb integer; specify three bands to form an rgb composite
+#' @param rgb integer; specify three bands to form an rgb composite. Experimental: rgb color table; see Details.
 #' @param maxColorValue numeric; passed on to \link{rgb}
 #' @param xlab character; x axis label
 #' @param ylab character; y axis label
@@ -161,69 +176,129 @@ get_breaks = function(x, breaks, nbreaks) {
 #' @param ylim y axis limits
 #' @param text_values logical; print values as text on image?
 #' @param interpolate logical; when using \link{rasterImage} (rgb), should pixels be interpolated?
+#' @param as_points logical; for curvilinear or sheared grids: parameter passed on to \link{st_as_sf}, determining whether raster cells will be plotted as symbols (fast, approximate) or small polygons (slow, exact)
+#' @param logz logical; if \code{TRUE}, use log10-scale for the attribute variable. In that case, \code{breaks} and \code{at} need to be given as log10-values; see examples.
+#' @details use of an rgb color table is experimental; see https://github.com/r-spatial/mapview/issues/208
 #' @export
 #' @examples
 #' tif = system.file("tif/L7_ETMs.tif", package = "stars")
 #' x = read_stars(tif)
 #' image(x, col = grey((3:9)/10))
 #' image(x, rgb = c(1,3,5)) # rgb composite
-image.stars = function(x, ..., band = 1, attr = 1, asp = NULL, rgb = NULL, maxColorValue = max(x[[attr]]),
-		xlab = if (!axes) "" else names(dims)[1], ylab = if (!axes) "" else names(dims)[2],
+image.stars = function(x, ..., band = 1, attr = 1, asp = NULL, rgb = NULL, 
+		maxColorValue = ifelse(inherits(rgb, "data.frame"), 255, max(x[[attr]], na.rm = TRUE)),
+		xlab = if (!axes) "" else names(d)[1], ylab = if (!axes) "" else names(d)[2],
 		xlim = st_bbox(x)$xlim, ylim = st_bbox(x)$ylim, text_values = FALSE, axes = FALSE,
-		interpolate = FALSE) {
+		interpolate = FALSE, as_points = FALSE, key.pos = NULL, logz = FALSE,
+		key.width = lcm(1.8), key.length = 0.618) {
 
 	dots = list(...)
-	stopifnot(!has_rotate_or_shear(x)) # FIXME: use rasterImage() with rotate, if only rotate & no shear
+
+	#stopifnot(!has_rotate_or_shear(x)) # FIXME: use rasterImage() with rotate, if only rotate & no shear
 
 	if (any(dim(x) == 1))
 		x = adrop(x)
 
 	force(xlim)
 	force(ylim)
-	dims = expand_dimensions(x)
 
-	y_is_neg = all(diff(dims$y) < 0)
-	if (y_is_neg)
-		dims[[2]] = rev(dims[[2]])
+	d = st_dimensions(x)
 
-	if (is.null(asp)) {
-		bb = st_bbox(x)
-		asp <- ifelse(isTRUE(st_is_longlat(x)), 1/cos((mean(bb[c(2,4)]) * pi)/180), 1.0)
+	dimxy = attr(d, "raster")$dimensions
+	dimx =  dimxy[1]
+	dimy =  dimxy[2]
+	dimxn = match(dimx, names(d))
+	dimyn = match(dimy, names(d))
+
+	if (has_sfc(x) && length(dim(x)) == 2) { # time series of features:
+		ed = lapply(expand_dimensions(x), function(x) if (inherits(x, "sfc")) seq_along(x) else x)
+		names(ed) = c("x", "y")
+		ed$z = x[[attr]]
+		axes = TRUE
+		return(image(ed, ..., xlab = xlab, ylab = ylab))
 	}
 
+	if (! is_curvilinear(x)) {
+		dims = expand_dimensions.stars(x, center = FALSE, max = FALSE)
+		d_max = expand_dimensions.stars(x, center = FALSE, max = TRUE)
+		if (tail(dims[[dimx]], 1) != tail(d_max[[dimx]], 1))
+			dims[[ dimx ]] = c(dims[[dimx]], tail(d_max[[dimx]], 1))
+		if (tail(dims[[dimy]], 1) != tail(d_max[[dimy]], 1))
+			dims[[ dimy ]] = c(dims[[dimy]], tail(d_max[[dimy]], 1))
+
+		y_is_neg = all(diff(dims[[ dimy ]]) < 0)
+		if (y_is_neg)
+			dims[[ dimy ]] = rev(dims[[ dimy ]])
+	}
+
+	if (is.null(asp))
+		asp = if (isTRUE(st_is_longlat(x))) {
+				bb = st_bbox(x)
+				1 / cos((mean(bb[c(2,4)]) * pi)/180)
+			} else
+				1.0
+
 	ar = unclass(x[[ attr ]]) # raw data matrix/array
-	if (!is.null(rgb)) {
-		if (is_rectilinear(x))
-			warning("rectilinear rgb grid is plotted as regular grid")
+
+	# rearrange ar:
+	others = setdiff(seq_along(dim(ar)), c(dimxn, dimyn))
+	ar = aperm(ar, c(dimxn, dimyn, others))
+
+	if (! is.null(rgb)) {
+		if (is_curvilinear(x))
+			warning("when using rgb, curvilinear grid is plotted as regular grid")
 		xy = dim(ar)[1:2]
-		ar = structure(ar[ , , rgb], dim = c(prod(xy), 3)) # flattens x/y
-		nas = apply(ar, 1, function(x) any(is.na(x)))
-		ar = rgb(ar[!nas,], maxColorValue = maxColorValue)
-		mat = rep(NA_character_, prod(xy))
-		mat[!nas] = ar
-		dim(mat) = xy
+		if (!y_is_neg) # need to flip y?
+			ar = ar[ , rev(seq_len(dim(ar)[2])), ]
 		if (dev.capabilities("rasterImage")$rasterImage != "yes")
 			stop("rgb plotting not supported on this device")
 		if (! isTRUE(dots$add)) {
 			plot.new()
 			plot.window(xlim = xlim, ylim = ylim, asp = asp)
 		}
-		myRasterImage = function(x, xmin, ymin, xmax, ymax, interpolate, ..., breaks, add) # absorbs breaks & add
+		if (is.numeric(rgb) && length(rgb) == 3) {
+			ar = structure(ar[ , , rgb], dim = c(prod(xy), 3)) # flattens x/y
+			nas = apply(ar, 1, function(x) any(is.na(x)))
+			ar = grDevices::rgb(ar[!nas,], maxColorValue = maxColorValue)
+			mat = rep(NA_character_, prod(xy))
+			mat[!nas] = ar
+			dim(mat) = xy
+		} else {
+			stopifnot(inherits(rgb, "data.frame"))
+			# rgb has col 1: index, col 2: label, col 3-5: R, G, B
+			#ar = as.vector(ar[ , , 1]) # flattens x/y to 1-D index vector
+			ar = as.vector(ar) # flattens x/y to 1-D index vector
+			rgb = grDevices::rgb(rgb[match(ar, rgb[[1]]), 3:5], maxColorValue = maxColorValue)
+			mat = structure(rgb, dim = xy)
+		}
+		myRasterImage = function(x, xmin, ymin, xmax, ymax, interpolate, ..., breaks, add, zlim) # absorbs breaks, add & zlim
 			rasterImage(x, xmin, ymin, xmax, ymax, interpolate = interpolate, ...)
 		myRasterImage(t(mat), xlim[1], ylim[1], xlim[2], ylim[2], interpolate = interpolate, ...)
-	} else { 
-		if (y_is_neg) {
-			ar = if (length(dim(x)) == 3) # FIXME: deal with more than 3 dims here?
-				ar[ , rev(seq_len(dim(ar)[2])), band]
+	} else if (is_curvilinear(x) || has_rotate_or_shear(x)) {
+		x = st_as_sf(x[1], as_points = as_points)
+		mplot = function(x, col, ..., zlim) {
+			if (missing(col))
+				plot(x, ...)
 			else
-				ar[ , rev(seq_len(dim(ar)[2]))]
+				plot(x, pal = col, ...) # need to swap arg names: FIXME:?
 		}
-		image.default(dims[[1]], dims[[2]], unclass(ar), asp = asp, xlab = xlab, ylab = ylab, 
-			xlim = xlim, ylim = ylim, axes = FALSE,...)
+		mplot(x, xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim, axes = axes, reset = FALSE, 
+			key.pos = key.pos, key.width = key.width, key.length = key.length, logz = logz, ...)
+	} else { # regular grid, no RGB:
+		if (y_is_neg) { # need to flip y?
+			ar = if (length(dim(ar)) == 2)
+					ar[ , rev(seq_len(dim(ar)[2]))]
+				else
+					ar[ , rev(seq_len(dim(ar)[2])), band] # FIXME: breaks if more than 3?
+		}
+		image.default(dims[[ dimx ]], dims[[ dimy ]], ar, asp = asp, xlab = xlab, ylab = ylab, 
+			xlim = xlim, ylim = ylim, axes = FALSE, ...)
 	}
-	if (text_values)
+	if (text_values) {
+		dims = expand_dimensions.stars(x, center = TRUE)
 		text(do.call(expand.grid, dims[1:2]), labels = as.character(as.vector(ar))) # xxx
-	if (axes) {
+	}
+	if (axes) { # FIXME: see sf::plot.sf for refinements to be ported here?
         if (isTRUE(st_is_longlat(x))) {
             .degAxis(1)
             .degAxis(2)
@@ -237,13 +312,16 @@ image.stars = function(x, ..., band = 1, attr = 1, asp = NULL, rgb = NULL, maxCo
 # reduce resolution of x, keeping (most of) extent
 st_downsample = function(x, n) {
 	stopifnot(all(n >= 0))
-	d = dim(x)
-	n = rep(n, length.out = length(d))
-	args = rep(list(rlang::missing_arg()), length(d)+1)
-	for (i in seq_along(d))
-		if (n[i] > 1)
-			args[[i+1]] = seq(1, d[i], n[i])
-	eval(rlang::expr(x[!!!args]))
+	if (! all(n <= 1)) {
+		d = dim(x)
+		n = rep(n, length.out = length(d))
+		args = rep(list(rlang::missing_arg()), length(d)+1)
+		for (i in seq_along(d))
+			if (n[i] > 1)
+				args[[i+1]] = seq(1, d[i], n[i])
+		eval(rlang::expr(x[!!!args]))
+	} else
+		x
 }
 
 # compute the degree of downsampling allowed to still have more than 
